-- ENV
-- {{ SCHEMA_NAME }}
-- {{ TABLE_NAME }}

DROP TABLE IF EXISTS {{ SCHEMA_NAME }}."{{ TABLE_NAME }}";

-- remove the batch from hot reload info table. 
DELETE FROM public.hot_reload
WHERE EXISTS (SELECT FROM public.hot_reload
    WHERE simulation = '{{ SCHEMA_NAME }}'
        and batch = '{{ TABLE_NAME }}'
);